__doc__='''ZCache'''
__version__='1.0'

from Globals import HTMLFile, MessageDialog, package_home
import OFS.SimpleItem, Acquisition, AccessControl.Role
from BoboPOS import Persistence
import sys, time

def manage_addZCache(self, id, cacheObject, delay, REQUEST=None):
    """Add ZCache Object"""
    i=ZCache(id, cacheObject, delay, REQUEST)
    self._setObject(id,i)
    if REQUEST is not None:
	return self.manage_main(self,REQUEST)

manage_addZCacheForm=HTMLFile('zcacheAdd', globals())

class ZCache(
    OFS.SimpleItem.Item,
    Persistence.Persistent,
    Acquisition.Implicit,
    AccessControl.Role.RoleManager,
    ):
    """ZCache object - store self.cacheObject's results in memory, and return it when called, 
     updating the stored value at most every self.delay seconds"""
    meta_type='ZCache'

    icon = 'zcache'

    manage=manage_main=HTMLFile('manageZCache', globals())

    manage_options=({'icon':'', 'label':'Properties',
                    'action':'manage_main', 'target':'manage_main'},
		    {'icon':'', 'label':'View',
                    'action':'index_html', 'target':'manage_main'})
			 
    __ac_permissions__=(
	        ('View management screens', ('manage', 'manage_main',)),
		('Change configuration', ('manage_makeChanges',)),
		)
					
    
    def __init__(self, id, cacheObject, delay, REQUEST):
	"""Create a ZCache - pass object to cache and minimum delay between reads (in seconds) """
	self.id	  = id
	self.cacheObject = cacheObject
	self.lastreadtime = 0
	self.cache = None
	self.delay = delay
    
    def index_html(self, REQUEST):
	"""Return contents of cache, updating if hasn't been updated for more that self.delay seconds"""
	now = time.time()
	if (self.cache is not None) and (now > self.lastreadtime + self.delay):
	    self.cache = None
	if self.cache is None:
	    self.update(REQUEST)
	return self.cache

    def update(self, REQUEST):
	""" Force update of contents of cache """ 
	try: self.cache = self.aq_parent[self.cacheObject](self, REQUEST)
	except: raise sys.exc_type, sys.exc_value
	self.lastreadtime = time.time()
	
    def manage_makeChanges(self, cacheObject, delay, REQUEST=None):
	""" Change properties """
	self.cacheObject = cacheObject
	self.delay = delay
	self.update(REQUEST)
	if REQUEST: return MessageDialog(title  ='Changed %s' % self.__name__,
					 message='%s has been updated' % self.id,
				         action =REQUEST['URL2']+'/manage_main',
					 target ='manage_main')